diff --git a/modules/gui/macosx/controls.m b/modules/gui/macosx/controls.m
index c042eb9..b4cf758 100644
--- a/modules/gui/macosx/controls.m
+++ b/modules/gui/macosx/controls.m
@@ -246,9 +246,9 @@
     playlist_t *p_playlist = pl_Hold( VLCIntf );
     var_Get( p_playlist, "random", &val );
     [o_btn_shuffle setState: val.b_bool];
-	if(val.b_bool)
+    if(val.b_bool)
         [o_btn_shuffle_embed setImage: [NSImage imageNamed:@"sidebarShuffleOn"]];
-	else
+    else
         [o_btn_shuffle_embed setImage: [NSImage imageNamed:@"sidebarShuffle"]];    
     pl_Release( VLCIntf );
 }
@@ -732,15 +732,15 @@
     switch( i_type & VLC_VAR_TYPE )
     {
     case VLC_VAR_VOID:
-        o_data = [[VLCAutoGeneratedMenuContent alloc] initWithVariableName: psz_variable ofObject: p_object
-                andValue: val ofType: i_type];
-        [o_mi setRepresentedObject: [o_data autorelease]];
+        o_data = [[VLCAutoGeneratedMenuContent alloc] initWithVariableName:psz_variable ofObject:p_object andValue:val ofType:i_type];
+        [o_mi setRepresentedObject: o_data];
+        [o_data release];
         break;
 
     case VLC_VAR_BOOL:
-        o_data = [[VLCAutoGeneratedMenuContent alloc] initWithVariableName: psz_variable ofObject: p_object
-                andValue: val ofType: i_type];
-        [o_mi setRepresentedObject: [o_data autorelease]];
+        o_data = [[VLCAutoGeneratedMenuContent alloc] initWithVariableName:psz_variable ofObject:p_object andValue:val ofType:i_type];
+        [o_mi setRepresentedObject: o_data];
+        [o_data release];
         if( !( i_type & VLC_VAR_ISCOMMAND ) )
             [o_mi setState: val.b_bool ? TRUE : FALSE ];
         break;
@@ -910,6 +910,7 @@
 {
     vlc_object_t *p_object;
     NSAutoreleasePool * o_pool = [[NSAutoreleasePool alloc] init];
+    int result = VLC_EGENERIC;
 
     assert([data isKindOfClass:[VLCAutoGeneratedMenuContent class]]);
     VLCAutoGeneratedMenuContent *menuContent = (VLCAutoGeneratedMenuContent *)data;
@@ -922,11 +923,10 @@
     {
         var_Set( p_object, [menuContent name], [menuContent value] );
         vlc_object_release( p_object );
-        [o_pool release];
-        return true;
+        result = true;
     }
-    [o_pool release];
-    return VLC_EGENERIC;
+    [o_pool drain];
+    return result;
 }
 
 - (IBAction)goToSpecificTime:(id)sender
@@ -1138,7 +1138,7 @@
     if( [[o_mi title] isEqualToString: _NS("Normal Size")] )
     {
         NSMenuItem *item = [[o_mi menu] itemWithTitle:_NS("Teletext")];
-		bool b_telx = p_input && var_GetInteger( p_input, "teletext-es" ) >= 0;
+        bool b_telx = p_input && var_GetInteger( p_input, "teletext-es" ) >= 0;
 
         [[item submenu] setAutoenablesItems:NO];
         for( int k=0; k < [[item submenu] numberOfItems]; k++ )
@@ -1171,22 +1171,34 @@
     if( self != nil )
     {
         _vlc_object = vlc_object_hold( object );
+        msg_Dbg(VLCIntf, "VLCAutoGeneratedMenuContent: holding vlc object %p", _vlc_object);
         psz_name = strdup( name );
+        msg_Dbg(VLCIntf, "VLCAutoGeneratedMenuContent: menu name is %s and type %d", name, type);
         i_type = type;
         value = val;
         if( (i_type & VLC_VAR_TYPE) == VLC_VAR_STRING )
             value.psz_string = strdup( val.psz_string );
     }
+    else
+    {
+        msg_Dbg(VLCIntf, "ERROR: self equals nil in initWithVariableName of VLCAutoGeneratedMenuContent, vlc object %p, name = %s", object, name);
+        assert(false);
+    }
 
-    return( self );
+    return self;
 }
 
 - (void)dealloc
 {
+    assert( _vlc_object != NULL ); // attempting to release a NULL pointer?
+    msg_Dbg(VLCIntf, "VLCAutoGeneratedMenuContent: releasing vlc object %p, name %s", _vlc_object, psz_name);
     vlc_object_release( _vlc_object );
+    _vlc_object = NULL; // setup the object so that if a double dealloc is called the assertion will trigger
+
     if( (i_type & VLC_VAR_TYPE) == VLC_VAR_STRING )
         free( value.psz_string );
     free( psz_name );
+
     [super dealloc];
 }
 
diff --git a/modules/gui/macosx/intf.m b/modules/gui/macosx/intf.m
index 2d44438..aa9cf82 100644
--- a/modules/gui/macosx/intf.m
+++ b/modules/gui/macosx/intf.m
@@ -76,6 +76,9 @@ static void destroyProgressPanel (void *);
 
 static void MsgCallback( msg_cb_data_t *, msg_item_t *, unsigned );
 
+static NSLock * o_terminate_lock = nil;
+static int terminate_signaled = 0;
+
 #pragma mark -
 #pragma mark VLC Interface Object Callbacks
 
@@ -113,7 +116,6 @@ void CloseIntf ( vlc_object_t *p_this )
 /*****************************************************************************
  * Run: main loop
  *****************************************************************************/
-jmp_buf jmpbuffer;
 
 static void Run( intf_thread_t *p_intf )
 {
@@ -134,19 +136,14 @@ static void Run( intf_thread_t *p_intf )
 
     NSAutoreleasePool * o_pool = [[NSAutoreleasePool alloc] init];
 
-    /* Install a jmpbuffer to where we can go back before the NSApp exit
-     * see applicationWillTerminate: */
     [VLCApplication sharedApplication];
 
     [[VLCMain sharedInstance] setIntf: p_intf];
     [NSBundle loadNibNamed: @"MainMenu" owner: NSApp];
 
-    /* Install a jmpbuffer to where we can go back before the NSApp exit
-     * see applicationWillTerminate: */
-    if(setjmp(jmpbuffer) == 0)
-        [NSApp run];
+    [NSApp run];
     
-    [o_pool release];
+    [o_pool drain];
 }
 
 #pragma mark -
@@ -175,7 +172,7 @@ static void MsgCallback( msg_cb_data_t *data, msg_item_t *item, unsigned int i )
                                                         object: nil 
                                                       userInfo: o_dict];
 
-    [o_pool release];
+    [o_pool drain];
     vlc_restorecancel( canc );
 }
 
@@ -247,7 +244,7 @@ static int DialogCallback( vlc_object_t *p_this, const char *type, vlc_value_t p
     NSValue *o_value = [NSValue valueWithPointer:value.p_address];
     [[NSNotificationCenter defaultCenter] postNotificationName: @"VLCNewCoreDialogEventNotification" object:[interface coreDialogProvider] userInfo:[NSDictionary dictionaryWithObjectsAndKeys: o_value, @"VLCDialogPointer", [NSString stringWithUTF8String: type], @"VLCDialogType", nil]];
 
-    [o_pool release];
+    [o_pool drain];
     return VLC_SUCCESS;
 }
 
@@ -263,21 +260,21 @@ void updateProgressPanel (void *priv, const char *text, float value)
 
     [[[VLCMain sharedInstance] coreDialogProvider] updateProgressPanelWithText: o_txt andNumber: (double)(value * 1000.)];
 
-    [o_pool release];
+    [o_pool drain];
 }
 
 void destroyProgressPanel (void *priv)
 {
     NSAutoreleasePool *o_pool = [[NSAutoreleasePool alloc] init];
     [[[VLCMain sharedInstance] coreDialogProvider] destroyProgressPanel];
-    [o_pool release];
+    [o_pool drain];
 }
 
 bool checkProgressPanel (void *priv)
 {
     NSAutoreleasePool *o_pool = [[NSAutoreleasePool alloc] init];
     return [[[VLCMain sharedInstance] coreDialogProvider] progressCancelled];
-    [o_pool release];
+    [o_pool drain];
 }
 
 #pragma mark -
@@ -774,13 +771,13 @@ static VLCMain *_o_sharedMainInstance = nil;
     if( !p_intf ) return;
 
     NSArray *menuitems_array = [the_menu itemArray];
-    for( int i=0; i<[menuitems_array count]; i++ )
+    for( int i = [menuitems_array count] - 1; i >= 0; i-- )
     {
         NSMenuItem *one_item = [menuitems_array objectAtIndex: i];
         if( [one_item hasSubmenu] )
             [self releaseRepresentedObjects: [one_item submenu]];
 
-        [one_item setRepresentedObject:NULL];
+        [one_item setRepresentedObject:nil];
     }
 }
 
@@ -795,13 +792,21 @@ static VLCMain *_o_sharedMainInstance = nil;
     msg_Dbg( p_intf, "Terminating" );
 
     /* Make sure the manage_thread won't call -terminate: again */
-    pthread_cancel( manage_thread );
+    //pthread_cancel( manage_thread );
 
     /* Make sure the intf object is getting killed */
+    msg_Dbg(VLCIntf, "sending kill to %p", p_intf);
     vlc_object_kill( p_intf );
+    msg_Dbg(VLCIntf, "terminating GUI thread");
 
-    /* Make sure our manage_thread ends */
+    [o_terminate_lock lock];
+    terminate_signaled = 1;
+    [o_terminate_lock unlock];
     pthread_join( manage_thread, NULL );
+    [o_terminate_lock release];
+
+    /* Make sure our manage_thread ends */
+    //pthread_join( manage_thread, NULL );
 
     /* Make sure the interfaceTimer is destroyed */
     [interfaceTimer invalidate];
@@ -895,10 +900,6 @@ static VLCMain *_o_sharedMainInstance = nil;
 
     [self setIntf:nil];
 
-    /* Go back to Run() and make libvlc exit properly */
-    if( jmpbuffer )
-        longjmp( jmpbuffer, 1 );
-    /* not reached */
 }
 
 #pragma mark -
@@ -1024,8 +1025,8 @@ static NSString * VLCToolbarMediaControl     = @"VLCToolbarMediaControl";
 - (void)applicationDidBecomeActive:(NSNotification *)aNotification
 {
     if( !p_intf ) return;
-	if( config_GetInt( p_intf, "macosx-appleremote" ) == YES )
-		[o_remote startListening: self];
+    if( config_GetInt( p_intf, "macosx-appleremote" ) == YES )
+        [o_remote startListening: self];
 }
 - (void)applicationDidResignActive:(NSNotification *)aNotification
 {
@@ -1510,7 +1511,7 @@ static unsigned int VLCModifiersToCocoa( unsigned int i_key )
 
 - (id)appleRemoteController
 {
-	return o_remote;
+    return o_remote;
 }
 
 #pragma mark -
@@ -1523,6 +1524,9 @@ static void * ManageThread( void *user_data )
 {
     id self = user_data;
 
+    o_terminate_lock    = [[NSLock alloc] init];
+    terminate_signaled = 0;
+
     [self manage];
 
     return NULL;
@@ -1574,8 +1578,10 @@ static void manage_cleanup( void * args )
     struct manage_cleanup_stack stack = { p_intf, &p_input, p_playlist, self };
     pthread_cleanup_push(manage_cleanup, &stack);
 
-    while( true )
+    int stop = 0;
+    while( !stop )
     {
+    
         NSAutoreleasePool * pool = [[NSAutoreleasePool alloc] init];
 
         if( !p_input )
@@ -1606,9 +1612,13 @@ static void manage_cleanup( void * args )
         /* Manage volume status */
         [self manageVolumeSlider];
 
+        [pool release];
+
         msleep( INTF_IDLE_SLEEP );
 
-        [pool release];
+        [o_terminate_lock lock];
+        stop = terminate_signaled;
+        [o_terminate_lock unlock];
     }
 
     pthread_cleanup_pop(1);
@@ -2426,7 +2436,7 @@ end:
     NSString * latestLog = [self latestCrashLogPathPreviouslySeen:NO];
     if( latestLog && !areCrashLogsTooOld )
         [NSApp runModalForWindow: o_crashrep_win];
-    [o_pool release];
+    [o_pool drain];
 }
 
 - (IBAction)crashReporterAction:(id)sender
@@ -2858,7 +2868,7 @@ end:
 
 - (void)awakeFromNib
 {
-	b_active = b_mediaKeySupport = config_GetInt( VLCIntf, "macosx-mediakeys" );
+    b_active = b_mediaKeySupport = config_GetInt( VLCIntf, "macosx-mediakeys" );
     b_activeInBackground = config_GetInt( VLCIntf, "macosx-mediakeys-background" );
     [[NSNotificationCenter defaultCenter] addObserver: self selector: @selector(coreChangedMediaKeySupportSetting:) name: @"VLCMediaKeySupportSettingChanged" object: nil];
     [[NSNotificationCenter defaultCenter] addObserver: self selector: @selector(appGotActiveOrInactive:) name: @"NSApplicationDidBecomeActiveNotification" object: nil];
@@ -2889,7 +2899,7 @@ end:
 - (void)sendEvent: (NSEvent*)event
 {
     if( b_active )
-	{
+    {
         if( [event type] == NSSystemDefined && [event subtype] == 8 )
         {
             int keyCode = (([event data1] & 0xFFFF0000) >> 16);
@@ -2933,7 +2943,7 @@ end:
             }
         }
     }
-	[super sendEvent: event];
+    [super sendEvent: event];
 }
 
 - (void)resetJump
diff --git a/modules/gui/macosx/vout.m b/modules/gui/macosx/vout.m
index f333b20..757b765 100644
--- a/modules/gui/macosx/vout.m
+++ b/modules/gui/macosx/vout.m
@@ -249,7 +249,7 @@ int DeviceCallback( vlc_object_t *p_this, const char *psz_variable,
     /* Add the view. It's automatically resized to fit the window */
     [self addSubview: o_view];
     [self setAutoresizesSubviews: YES];
-    [o_pool release];
+    [o_pool drain];
 
     return YES;
 }
diff --git a/modules/gui/macosx/voutgl.m b/modules/gui/macosx/voutgl.m
index 7cf4992..1530487 100644
--- a/modules/gui/macosx/voutgl.m
+++ b/modules/gui/macosx/voutgl.m
@@ -181,7 +181,7 @@ int OpenVideoGL  ( vlc_object_t * p_this )
 
         [VLCGLView performSelectorOnMainThread:@selector(initVout:) withObject:[NSValue valueWithPointer:p_vout] waitUntilDone:YES];
 
-        [o_pool release];
+        [o_pool drain];
 
         /* Check to see if initVout: was successfull */
         if( !p_vout->p_sys->o_vout_view )
@@ -235,7 +235,7 @@ void CloseVideoGL ( vlc_object_t * p_this )
         /* Close the window */
         [p_vout->p_sys->o_vout_view performSelectorOnMainThread:@selector(closeVout) withObject:NULL waitUntilDone:YES];
 
-        [o_pool release];
+        [o_pool drain];
     }
     /* Clean up */
     free( p_vout->p_sys );
@@ -276,7 +276,7 @@ static int Manage( vout_thread_t * p_vout )
         else
             [p_vout->p_sys->o_vout_view leaveFullscreen];
 
-        [o_pool release];
+        [o_pool drain];
 
         p_vout->i_changes &= ~VOUT_FULLSCREEN_CHANGE;
     }
diff --git a/src/misc/objects.c b/src/misc/objects.c
index fa3805e..d0fcb6a 100644
--- a/src/misc/objects.c
+++ b/src/misc/objects.c
@@ -603,6 +603,12 @@ void __vlc_object_release( vlc_object_t *p_this )
             vlc_object_detach_unlocked (p_this);
 
         /* We have no children */
+        if (internals->i_children) {
+            msg_Dbg(p_this, "assert should follow: internals->i_children == %d", internals->i_children);
+            msg_Dbg(p_this, "children are...");
+            for (int ndx = 0; ndx < internals->i_children; ndx++)
+                msg_Dbg(p_this, "child[%d] = %p", ndx, internals->pp_children[ndx]);
+        }
         assert (internals->i_children == 0);
     }
     libvlc_unlock (p_this->p_libvlc);
